unit UnitSecond;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.Gestures,
  FMXTee.Engine, FMXTee.Series, FMXTee.Procs, FMXTee.Chart, FMX.StdCtrls,
  FMX.Controls.Presentation;

type
  TSecondForm = class(TForm)
    ToolBar1: TToolBar;
    SpeedButton1: TSpeedButton;
    Chart1: TChart;
    SeriesLeft: TLineSeries;
    SeriesRight: TLineSeries;
    GestureManager1: TGestureManager;
    procedure Chart1Gesture(Sender: TObject; const EventInfo: TGestureEventInfo;
      var Handled: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
  private
    { Private-Deklarationen }
    FLastPosition: TPointF;
    FLastDistance: Integer;
    SeriesAxis: TLineSeries;
    procedure handlePan(eventInfo: TGestureEventInfo);
    procedure handleZoom(eventInfo: TGestureEventInfo);
    procedure handleDoubletap(eventInfo: TGestureEventInfo);
    procedure Series1Actualize();
    procedure PaintAxis();
  public
    { Public-Deklarationen }
    procedure ScaleMinMax;
  end;

var
  SecondForm: TSecondForm;

implementation

{$R *.fmx}

uses UnitMain, System.Math;

procedure TSecondForm.Chart1AfterDraw(Sender: TObject);
begin
  PaintAxis;
end;

procedure TSecondForm.Chart1Gesture(Sender: TObject;
  const EventInfo: TGestureEventInfo; var Handled: Boolean);
begin
  if EventInfo.GestureID = igiZoom then
      handleZoom(EventInfo);
  if EventInfo.GestureID = igiPan then
     handlePan(EventInfo);
  if EventInfo.GestureID = igiDoubleTap then
     handleDoubleTap(EventInfo);

  Handled:=True;
end;

procedure TSecondForm.FormCreate(Sender: TObject);
begin
  SeriesLeft.SeriesColor:=TAlphaColorRec.Black;
  SeriesLeft.Pointer.Visible:=True;

  SeriesRight.SeriesColor:=TAlphaColorRec.Black;
  SeriesRight.Pointer.Visible:=True;

  SeriesAxis:=TLineSeries.Create(Chart1);
  SeriesAxis.Color:=TAlphaColorRec.Red;
  SeriesAxis.Pointer.Visible:=False;
  SeriesAxis.ValueFormat:='#.###';
  Chart1.AddSeries(SeriesAxis);

  Chart1.LeftAxis.Inverted:=True;

  Chart1.Zoom.Allow:=False;
  Chart1.Panning.Active:=False;
  Chart1.Title.Font.Size:=20;
  Chart1.legend.Visible:=False;
end;

procedure TSecondForm.FormShow(Sender: TObject);
begin
  Series1Actualize;
  ScaleMinMax;
end;

procedure TSecondForm.Series1Actualize;
begin
  SeriesLeft.Clear;
  SeriesLeft.AddXY(-1.5,500.33);
  SeriesLeft.AddXY(-2.1,505.33);
  SeriesLeft.AddXY(-3.5,510.33);
  SeriesLeft.AddXY(-4.5,550.33);
  SeriesLeft.AddXY(-6.5,600.33);

  SeriesRight.Clear;
  SeriesRight.AddXY(1.5,500.33);
  SeriesRight.AddXY(2.1,505.33);
  SeriesRight.AddXY(3.5,510.33);
  SeriesRight.AddXY(4.5,550.33);
  SeriesRight.AddXY(6.5,600.33);

  SeriesAxis.Clear;
  SeriesAxis.AddXY(0,0);
  SeriesAxis.AddXY(0,0);
end;

procedure TSecondForm.PaintAxis;
begin
  SeriesAxis.Clear;
  SeriesAxis.AddXY(0,Chart1.LeftAxis.minimum);
  SeriesAxis.AddXY(0,Chart1.LeftAxis.maximum);
end;

procedure TSecondForm.handlePan(EventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
  minimum,maximum,width,height: double;
  deltax,deltay:double;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    if not(TInteractiveGestureFlag.gfBegin in EventInfo.Flags) then
    begin
      chart := TChart(LObj.GetObject);
      chart.axes.left.Automatic:=False;
      chart.axes.bottom.Automatic:=False;
      // Set the X coordinate.
      minimum:=chart.axes.Bottom.minimum;
      maximum:=chart.axes.Bottom.maximum;
      Width:=chart.Axes.Bottom.CalcXPosValue(maximum)-chart.axes.bottom.calcxPosvalue(minimum);
      deltax:=(EventInfo.Location.X-FLastPosition.X)*(maximum-minimum)/Width;
      if deltax<>0 then
         chart.axes.bottom.SetMinMax(minimum-deltax,maximum-deltax);
      // Set the Y coordinate.
      minimum:=chart.axes.Left.minimum;
      maximum:=chart.axes.Left.maximum;
      Height:=chart.Axes.Left.CalcYPosValue(maximum)-chart.axes.Left.calcYPosvalue(minimum);
      deltay:=(EventInfo.Location.Y-FLastPosition.Y)*(maximum-minimum)/Height;
      if deltay<>0 then
         chart.axes.left.SetMinMax(minimum-deltay,maximum-deltay);
      PaintAxis;
    end;

    FLastPosition := EventInfo.Location;
  end;
end;

procedure TSecondForm.handleZoom(EventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
  minimum,maximum: double;
  deltax,deltay,faktor:double;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    if not(TInteractiveGestureFlag.gfBegin in EventInfo.Flags) and
       not(TInteractiveGestureFlag.gfEnd in EventInfo.Flags) then
    begin
      chart := TChart(LObj.GetObject);
      chart.axes.left.Automatic:=False;
      chart.axes.bottom.Automatic:=False;
      // Set the X coordinate.
      minimum:=chart.axes.Bottom.minimum;
      maximum:=chart.axes.Bottom.maximum;
      if EventInfo.Distance<FLastDistance then
         Faktor:=-1
      else
         Faktor:=1;
      deltax:=0;
      if abs(maximum-minimum)<1000 then
         deltax:=faktor*10;
      if abs(maximum-minimum)<100 then
         deltax:=faktor;
      if deltax<>0 then
      begin
        minimum:=minimum+deltax;
        maximum:=maximum-deltax;
        if abs(maximum-minimum)<1000*(SeriesLeft.MaxXValue-SeriesLeft.MinXValue) then
        begin
          if minimum>=maximum then
             maximum:=minimum+1;
          chart.axes.bottom.SetMinMax(minimum,maximum);
        end;
      end;
      // Set the Y coordinate.
      minimum:=chart.axes.Left.minimum;
      maximum:=chart.axes.Left.maximum;
      if EventInfo.Distance<FLastDistance then
         Faktor:=-1
      else
         Faktor:=1;
      deltay:=0;
      if abs(maximum-minimum)<1000 then
         deltay:=faktor*10;
      if abs(maximum-minimum)<100 then
         deltay:=faktor;
      if deltay<>0 then
      begin
        minimum:=minimum+deltay;
        maximum:=maximum-deltay;
        if abs(maximum-minimum)<1000*(SeriesLeft.MaxYValue-SeriesLeft.MinYValue) then
        begin
          if minimum>=maximum then
             maximum:=minimum+1;
          chart.axes.left.SetMinMax(minimum,maximum);
        end;
      end;
      PaintAxis;
    end;

    FLastDistance := EventInfo.Distance;
  end;
end;

procedure TSecondForm.handleDoubleTap(eventInfo: TGestureEventInfo);
var
  LObj: IControl;
  chart: TChart;
begin
  LObj := Self.ObjectAtPoint(ClientToScreen(EventInfo.Location));
  if LObj is TChart then
  begin
    chart:=(TChart(LObj.GetObject));
    if chart=Chart1 then
    begin
      ScaleMinMax;
    end;
  end;
end;

procedure TSecondForm.ScaleMinMax;
var
  MinX,MaxX,MinY,MaxY,DeltaX,DeltaY: double;
begin
  MinX:=SeriesLeft.MinXValue;
  MaxX:=SeriesLeft.MaxXValue;
  MinX:=min(MinX,SeriesRight.MinXValue);
  MaxX:=max(MaxX,SeriesRight.MaxXValue);
  DeltaX:=MaxX-MinX;
  Chart1.Axes.Bottom.SetMinMax(MinX-DeltaX/10,MaxX+DeltaX/10);
  MinY:=SeriesLeft.MinYValue;
  MaxY:=SeriesLeft.MaxYValue;
  MinY:=min(MinY,SeriesRight.MinYValue);
  MaxY:=max(MaxY,SeriesRight.MaxYValue);
  DeltaY:=MaxY-MinY;
  Chart1.Axes.Left.SetMinMax(MinY-DeltaY/10,MaxY+DeltaY/10);
  PaintAxis;
end;

procedure TSecondForm.SpeedButton1Click(Sender: TObject);
begin
  MainForm.Show;
end;

end.
